@@ -130,7 +130,7 @@ ptarray_insert_point(POINTARRAY *pa, const POINT4D *p, uint32_t where)
 	{
 		size_t copy_size = point_size * (pa->npoints - where);
 		memmove(getPoint_internal(pa, where+1), getPoint_internal(pa, where), copy_size);
-		LWDEBUGF(5,"copying %d bytes to start vertex %d from start vertex %d", copy_size, where+1, where);
+		LWDEBUGF(5,"copying %zu bytes to start vertex %d from start vertex %d", copy_size, where+1, where);
 	}
 
 	/* We have one more point */
@@ -138,7 +138,7 @@ ptarray_insert_point(POINTARRAY *pa, const POINT4D *p, uint32_t where)
 
 	/* Copy the new point into the gap */
 	ptarray_set_point4d(pa, where, p);
-	LWDEBUGF(5,"copying new point to start vertex %d", point_size, where);
+	LWDEBUGF(5,"copying new point to start vertex %zu", point_size);
 
 	return LW_SUCCESS;
 }
@@ -233,7 +233,7 @@ ptarray_append_ptarray(POINTARRAY *pa1, POINTARRAY *pa2, double gap_tolerance)
 		if ( pa1->maxpoints )
 		{
 			pa1->maxpoints = ncap > pa1->maxpoints*2 ?
-			                 ncap : pa1->maxpoints*2;
+											 ncap : pa1->maxpoints*2;
 			pa1->serialized_pointlist = lwrealloc(pa1->serialized_pointlist, (size_t)ptsize * pa1->maxpoints);
 		}
 		else
@@ -493,7 +493,7 @@ ptarray_same(const POINTARRAY *pa1, const POINTARRAY *pa2)
 	LWDEBUG(5,"npoints are the same");
 
 	ptsize = ptarray_point_size(pa1);
-	LWDEBUGF(5, "ptsize = %d", ptsize);
+	LWDEBUGF(5, "ptsize = %zu", ptsize);
 
 	for (i=0; i<pa1->npoints; i++)
 	{
@@ -533,7 +533,7 @@ ptarray_addPoint(const POINTARRAY *pa, uint8_t *p, size_t pdims, uint32_t where)
 	POINT4D pbuf;
 	size_t ptsize = ptarray_point_size(pa);
 
-	LWDEBUGF(3, "pa %x p %x size %d where %d",
+	LWDEBUGF(3, "pa %p p %p size %zu where %u",
 	         pa, p, pdims, where);
 
 	if ( pdims < 2 || pdims > 4 )
@@ -550,7 +550,7 @@ ptarray_addPoint(const POINTARRAY *pa, uint8_t *p, size_t pdims, uint32_t where)
 		return NULL;
 	}
 
-	LWDEBUG(3, "called with a %dD point");
+	LWDEBUG(3, "called with a point");
 
 	pbuf.x = pbuf.y = pbuf.z = pbuf.m = 0.0;
 	memcpy((uint8_t *)&pbuf, p, pdims*sizeof(double));
@@ -584,7 +584,7 @@ ptarray_removePoint(POINTARRAY *pa, uint32_t which)
 	POINTARRAY *ret;
 	size_t ptsize = ptarray_point_size(pa);
 
-	LWDEBUGF(3, "pa %x which %d", pa, which);
+	LWDEBUGF(3, "pa %p which %u", pa, which);
 
 	assert(which <= pa->npoints-1);
 	assert(pa->npoints >= 3);
@@ -768,6 +768,12 @@ ptarray_contains_point(const POINTARRAY *pa, const POINT2D *pt)
 
 		seg2 = getPoint2d_cp(pa, i);
 
+		/* This is required for robustness, see #6023 */
+		if (p2d_same(seg1, pt))
+		{
+			return LW_BOUNDARY;
+		}
+
 		/* Zero length segments are ignored. */
 		if (p2d_same(seg1, seg2))
 		{
@@ -1609,14 +1615,7 @@ ptarray_locate_point(const POINTARRAY *pa, const POINT4D *p4d, double *mindistou
 
 	LWDEBUGF(3, "plen %g, tlen %g", plen, tlen);
 
-	/* Floating point arithmetic is not reliable, make sure we return values [0,1] */
-	double result = plen / tlen;
-	if ( result < 0.0 ) {
-		result = 0.0;
-	} else if ( result > 1.0 ) {
-		result = 1.0;
-	}
-	return result;
+	return plen/tlen;
 }
 
 /**
@@ -1800,7 +1799,7 @@ ptarray_dp_findsplit_in_place(const POINTARRAY *pts, uint32_t it_first, uint32_t
 	return split;
 }
 
-/* O(N) simplification for tolearnce = 0 */
+/* O(N) simplification for tolerance = 0 */
 static void
 ptarray_simplify_in_place_tolerance0(POINTARRAY *pa)
 {
@@ -2226,7 +2225,7 @@ ptarray_startpoint(const POINTARRAY *pa, POINT4D *pt)
  *
  */
 void
-ptarray_grid_in_place(POINTARRAY *pa, const gridspec *grid)
+ptarray_grid_in_place(POINTARRAY *pa, gridspec *grid)
 {
 	uint32_t j = 0;
 	POINT4D *p, *p_out = NULL;
@@ -2246,11 +2245,24 @@ ptarray_grid_in_place(POINTARRAY *pa, const gridspec *grid)
 		if (ndims > 3)
 			m = p->m;
 
-		if (grid->xsize > 0)
-			x = rint((x - grid->ipx) / grid->xsize) * grid->xsize + grid->ipx;
+		/*
+		 * See https://github.com/libgeos/geos/pull/956
+		 * We use scale for rounding when gridsize is < 1 and
+		 * gridsize for rounding when scale < 1.
+		 */
+		if (grid->xsize > 0) {
+			if (grid->xsize < 1)
+				x = rint((x - grid->ipx) * grid->xscale) / grid->xscale + grid->ipx;
+			else
+				x = rint((x - grid->ipx) / grid->xsize) * grid->xsize + grid->ipx;
+		}
 
-		if (grid->ysize > 0)
-			y = rint((y - grid->ipy) / grid->ysize) * grid->ysize + grid->ipy;
+		if (grid->ysize > 0) {
+			if (grid->ysize < 1)
+				y = rint((y - grid->ipy) * grid->yscale) / grid->yscale + grid->ipy;
+			else
+				y = rint((y - grid->ipy) / grid->ysize) * grid->ysize + grid->ipy;
+		}
 
 		/* Read and round this point */
 		/* Z is always in third position */
@@ -2357,7 +2369,7 @@ ptarray_scroll_in_place(POINTARRAY *pa, const POINT4D *pt)
 	/* TODO: reduce allocations */
 	tmp = ptarray_construct(FLAGS_GET_Z(pa->flags), FLAGS_GET_M(pa->flags), pa->npoints);
 
-	memset(getPoint_internal(tmp, 0), 0,  (size_t)ptsize * pa->npoints);
+	memset(getPoint_internal(tmp, 0), 0, (size_t)ptsize * pa->npoints);
 	/* Copy the block from found point to last point into the output array */
 	memcpy(
 		getPoint_internal(tmp, 0),
