@@ -1,8 +1,9 @@
 set client_min_messages to WARNING;
 
-\i ../load_topology-4326.sql
+\i :top_builddir/topology/test/load_topology-4326.sql
 \i ../load_features.sql
 \i ../more_features.sql
+\i ../hierarchy.sql
 
 SELECT topology.CopyTopology('city_data', 'CITY_data_UP_down') > 0;
 
@@ -46,9 +47,11 @@ SELECT tableoid::regclass AS sequence_name, last_value,  is_called  from "CITY_d
 SELECT tableoid::regclass AS sequence_name, last_value,  is_called  from "CITY_data_UP_down".topogeo_s_2;
 SELECT tableoid::regclass AS sequence_name, last_value,  is_called  from "CITY_data_UP_down".topogeo_s_3;
 
-SELECT topology.DropTopology('CITY_data_UP_down');
-SELECT topology.DropTopology('city_data');
-DROP SCHEMA features CASCADE;
+-- See https://trac.osgeo.org/postgis/ticket/5298
+BEGIN;
+UPDATE city_data.relation SEt layer_id = 1 WHERE layer_id = 1;
+SELECT '#5298', topology.CopyTopology('city_data', 'city_data_t5298') > 0;
+ROLLBACK;
 
 -- See http://trac.osgeo.org/postgis/ticket/2184
 select '#2184.1', topology.createTopology('t3d', 0, 0, true) > 0;
@@ -56,3 +59,8 @@ select '#2184.2', st_addisonode('t3d', NULL, 'POINT(1 2 3)');
 select '#2184.3', topology.copyTopology('t3d', 't3d-bis') > 0;
 select '#2184.4', length(topology.dropTopology('t3d')) > 0, length(topology.dropTopology('t3d-bis')) > 0;
 
+-- Cleanup
+
+SELECT topology.DropTopology('CITY_data_UP_down');
+SELECT topology.DropTopology('city_data');
+DROP SCHEMA features CASCADE;
